unit FunDBEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, DBCtrls, ExtCtrls;

type
  TFunMode = (fmRandomSkip, fmSkipDup, fmRandomOver,
    fmAutoType);

  TFunDBEdit = class(TDBEdit)
  private
    FFunMode: TFunMode;
    fTimer: TTimer;
    FFrequencyPercent: Integer;
    procedure SetFunMode(const Value: TFunMode);
    procedure SetFrequencyPercent(const Value: Integer);
  protected
    procedure KeyPress (var Key: Char); override;
    procedure TimerProc (Sender: TObject);
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy (); override;
    procedure Loaded; override;
  published
    property FunMode: TFunMode
      read FFunMode write SetFunMode
      default fmRandomSkip;
    property FrequencyPercent: Integer
      read FFrequencyPercent write SetFrequencyPercent
      default 20;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('FunSide', [TFunDBEdit]);
end;

{ TFunDBEdit }

constructor TFunDBEdit.Create(AOwner: TComponent);
begin
  inherited Create (AOwner);
  FFunMode := fmRandomSkip;
  fTimer := TTimer.Create (nil);
  fTimer.Enabled := False;
  fTimer.OnTimer := TimerProc;
  FFrequencyPercent := 20;
end;

destructor TFunDBEdit.Destroy;
begin
  fTimer.Free;
  inherited Destroy;
end;

procedure TFunDBEdit.KeyPress(var Key: Char);
begin
  case FFunMode of
    fmRandomSkip:
      if Random (FFrequencyPercent) = 0 then
        Key := #0;
    fmSkipDup:
      if Pos (Key, Caption) > 0 then
        Key := #0;
    fmRandomOver:
      if Random (FFrequencyPercent) = 0 then
        PostMessage (Handle, wm_Char, Integer (Key), 0);
  end;
  inherited KeyPress (Key);
end;

procedure TFunDBEdit.Loaded;
begin
  inherited Loaded;
  fTimer.Interval := 1000 * FFrequencyPercent;
end;

procedure TFunDBEdit.SetFrequencyPercent(const Value: Integer);
begin
  if (Value < 2) or (Value > 100) then
    raise Exception.Create (
      'Frequency Percent should be a value between 2 and 100');
  FFrequencyPercent := Value;
  if not (csLoading in ComponentState) then
    fTimer.Interval := 1000 * Value;
end;

procedure TFunDBEdit.SetFunMode(const Value: TFunMode);
begin
  if FFunMode <> Value then
  begin
    if Value = fmAutoType then
      fTimer.Enabled := True;
    if FFunMode = fmAutoType then
      fTimer.Enabled := False;
    FFunMode := Value;
  end;
end;

procedure TFunDBEdit.TimerProc(Sender: TObject);
begin
  Caption := Caption + Char (32 + random (96));
end;

initialization
  Randomize;

end.
